# based on 
# OBJECT ALIGNER/DISPLACER SCRIPT
# Version 1.0
# by RobertT
# Author E-mail: rjtiess@warwick.net
# http://webusers.warwick.net/~u1019306/myblenderimages/aligndisplacescript.zip 
# 
# This script Release Date: Sept 19, 2009
# Author Guy Charbonneau
# Version 0.9

# This script Release Date: Sept 20, 2009
# Author Guy Charbonneau
# Version 0.91
# renamed all the buttons & added reset & added 3D cursor placements

# License: Go for it, except Warning and Terms of Service.
#
# WARNING/TERMS OF SERVICE: Use this at your own risk!
# The script has NOT been tested on all versions of
# Blender, nor does it come with any guarantees.  It
# is provided as is with no additional support.  Do not
# run this script on existing Blend files without a full
# understanding of how it works, as modifications cannot
# be undone.  A full Python installation is assumed.
# You have been warned.
#
# About This Script:
#
# Load and run this script in Blender (ALT P to run).
# This script enables you to align 2 or more objects.
# The bounding box of objects is used. 
#  So non-square shapes will align to chosen axis
#
# To align objects, select 2 or more objects in a 3D View.
# The last selected object's coordinates will be used
# to align the other objects. 
# You may align by X+/X-, Y+/Y-, Z+/Z-.
# The + implies the largest positive end of the object
# The - implies the smallest positive or largest negative end of the object.
# Example1:
#    X+ X+ will align the highest end to the highest end of the last object.
# Example2:
#    X- X+ will align the lowest  end to the largest end of the last object.
#
# Diagram: X+ to X+              X+ to X-                                             
#                                                                                
#  xxxxxxx  xxxxxxxxxxxxxxx               xxxxxxxxxxxxxxx                         
#  x     x  x last object x               x last object x                        
#  x     x  xxxxxxxxxxxxxxx      xxxxxxxx xxxxxxxxxxxxxxx                        
#  xxxxxxx                       x      x                                        
#                                xxxxxxxx                                        
#
# To reset an object implies setting all the vertex to +-0.5 
#  AND setScale(1,1,1) AND setEuler(0,0,0)
#
# To set cursor implies moving the 3D cursor to one of the bounding box edges of the object.
#  For visual clarity the cursor is positioned to the center of the chosen edge.
#
# 
# Clearly this is an experimental script, designed
# expressly for experimentation, and like all experiments
# some results are good, and others are not so good.
# The script doesn't modify your .blend file, only the
# active data in the view; HOWEVER, if you somehow save
# your blend file, any changes you made with or without
# the script will likely be irrevesible, so DO NOT SAVE
# YOUR FILE unless you're: A.) Happy with the results
# B.) Saving your file to a new file name (i.e. SAVE AS)
# so as to preserve your original .blend file should you
# need to go back to it at some point.
#
# So you can experiment safely and see what the script
# can do, a sample # scene (filename: sample.blend) has
# been included in this compressed archive
# (alignscript.zip).  Three files should be in
# the archive:  README.TXT, align.py (THIS FILE),
# and sample.blend.  If these three files are not present
# do not use this.  This archive should only be copied
# and republished in its original state.  When in doubt,
# download it again from the URL posted at the top of this
# notice.

import Blender
from Blender.Draw import *
from Blender.BGL import *
from Blender import Mesh
from Blender import Window
from Blender.Window import SetCursorPos

print "\n\nNow running: Object Aligner/Displacer Script (v1.0) by RobertT (rjtiess@warwick.net)"

#------------------------------------------------------------------------

def checkSelection(numberOfItemsSelected):
	objects = Blender.Object.GetSelected()
	if len(objects) < numberOfItemsSelected:
		print "\nError: Not enough objects selected."
		return False
	else:
		print "\nObject(s) moved.  Check results in 3D View."
		return True

def printOldCoordinates(objName):
		print "\n" , objName, "Old LocX:", objName.LocX, "Old LocY:", objName.LocY, "Old LocZ:" , objName.LocZ

def printNewCoordinates(objName):
		print "\n" , objName, "New LocX:", objName.LocX, "New LocY:", objName.LocY, "New LocZ:", objName.LocZ

def printOldSize(objName):
		print "\n" , objName, "Old SizeX:", objName.SizeX, "Old SizeY:", objName.SizeY, "Old SizeZ:" , objName.SizeZ

def printNewSize(objName):
		print "\n" , objName, "New SizeX:", objName.SizeX, "New SizeY:", objName.SizeY, "New SizeZ:", objName.SizeZ

def printOldRot(objName):
		print "\n" , objName, "Old RotX:", objName.RotX, "Old RotY:", objName.RotY, "Old RotZ:" , objName.RotZ

def printNewRot(objName):
		print "\n" , objName, "New RotX:", objName.RotX, "New RotY:", objName.RotY, "New RotZ:", objName.RotZ



#------------------------------------------------------------------------

#align bottom of objects to top of last object
def alignObjectBot2Top():
	checkSelection(2)
	o = Blender.Object.GetSelected()[0]
	sbb=o.getBoundBox(1)
	#calc miny and max y for last object
	sminy=50000
	smaxy=-50000
	for i in sbb:
		if (i[1] < sminy):
			sminy=i[1]
		if (i[1] > smaxy):
			smaxy=i[1]
	
	# for each other objs calc its miny and maxy
	# calc delta between obj and last obj
	# set objs locy to locy - delta
	objects = Blender.Object.GetSelected()
	for a in objects:
		if ( a != o):
			bb=a.getBoundBox(1)
			miny=50000
			maxy=-50000
			for i in bb:
				if (i[1] < miny):
					miny=i[1]
				if (i[1] > maxy):
					maxy=i[1]
			
			delta=miny - smaxy 
			a.setLocation (a.LocX, a.LocY - delta , a.LocZ) 
			printNewCoordinates(a)
	Blender.Redraw()

#align top of objects to top of last object
def alignObjectTop2Top():
	checkSelection(2)
	o = Blender.Object.GetSelected()[0]
	sbb=o.getBoundBox(1)
	#calc miny and max y for last object
	sminy=50000
	smaxy=-50000
	for i in sbb:
		if (i[1] < sminy):
			sminy=i[1]
		if (i[1] > smaxy):
			smaxy=i[1]
	
	# for each other objs calc its miny and maxy
	# calc delta between obj and last obj
	# set objs locy to locy - delta
	objects = Blender.Object.GetSelected()
	for a in objects:
		if ( a != o):
			bb=a.getBoundBox(1)
			miny=50000
			maxy=-50000
			for i in bb:
				if (i[1] < miny):
					miny=i[1]
				if (i[1] > maxy):
					maxy=i[1]
			
			delta=maxy - smaxy 
			a.setLocation (a.LocX, a.LocY - delta , a.LocZ) 
			printNewCoordinates(a)
	Blender.Redraw()

#align top of objects to top of last object
def alignObjectTop2Bot():
	checkSelection(2)
	o = Blender.Object.GetSelected()[0]
	sbb=o.getBoundBox(1)
	#calc miny and max y for last object
	sminy=50000
	smaxy=-50000
	for i in sbb:
		if (i[1] < sminy):
			sminy=i[1]
		if (i[1] > smaxy):
			smaxy=i[1]
	
	# for each other objs calc its miny and maxy
	# calc delta between obj and last obj
	# set objs locy to locy - delta
	objects = Blender.Object.GetSelected()
	for a in objects:
		if ( a != o):
			bb=a.getBoundBox(1)
			miny=50000
			maxy=-50000
			for i in bb:
				if (i[1] < miny):
					miny=i[1]
				if (i[1] > maxy):
					maxy=i[1]
			
			delta=maxy - sminy 
			a.setLocation (a.LocX, a.LocY - delta , a.LocZ) 
			printNewCoordinates(a)
	Blender.Redraw()

#align top of objects to top of last object
def alignObjectBot2Bot():
	checkSelection(2)
	o = Blender.Object.GetSelected()[0]
	sbb=o.getBoundBox(1)
	#calc miny and max y for last object
	sminy=50000
	smaxy=-50000
	for i in sbb:
		if (i[1] < sminy):
			sminy=i[1]
		if (i[1] > smaxy):
			smaxy=i[1]
	
	# for each other objs calc its miny and maxy
	# calc delta between obj and last obj
	# set objs locy to locy - delta
	objects = Blender.Object.GetSelected()
	for a in objects:
		if ( a != o):
			bb=a.getBoundBox(1)
			miny=50000
			maxy=-50000
			for i in bb:
				if (i[1] < miny):
					miny=i[1]
				if (i[1] > maxy):
					maxy=i[1]
			
			delta=miny - sminy 
			a.setLocation (a.LocX, a.LocY - delta , a.LocZ) 
			printNewCoordinates(a)
	Blender.Redraw()


#------------------------------------------------------------------------

#align left of objects to right of last object
def alignObjectLeft2Right():
	checkSelection(2)
	o = Blender.Object.GetSelected()[0]
	sbb=o.getBoundBox(1)
	#calc miny and max y for last object
	sminx=50000
	smaxx=-50000
	for i in sbb:
		if (i[0] < sminx):
			sminx=i[0]
		if (i[0] > smaxx):
			smaxx=i[0]
	
	# for each other objs calc its minx and maxx
	# calc delta between obj and last obj
	# set objs locx to locx - delta
	objects = Blender.Object.GetSelected()
	for a in objects:
		if ( a != o):
			bb=a.getBoundBox(1)
			minx=50000
			maxx=-50000
			for i in bb:
				if (i[0] < minx):
					minx=i[0]
				if (i[0] > maxx):
					maxx=i[0]
			
			delta=minx - smaxx 
			a.setLocation (a.LocX - delta, a.LocY, a.LocZ) 
			printNewCoordinates(a)
	Blender.Redraw()

#align right of objects to right of last object
def alignObjectRight2Right():
	checkSelection(2)
	o = Blender.Object.GetSelected()[0]
	sbb=o.getBoundBox(1)
	#calc miny and max y for last object
	sminx=50000
	smaxx=-50000
	for i in sbb:
		if (i[0] < sminx):
			sminx=i[0]
		if (i[0] > smaxx):
			smaxx=i[0]
	
	# for each other objs calc its minx and maxx
	# calc delta between obj and last obj
	# set objs locx to locx - delta
	objects = Blender.Object.GetSelected()
	for a in objects:
		if ( a != o):
			bb=a.getBoundBox(1)
			minx=50000
			maxx=-50000
			for i in bb:
				if (i[0] < minx):
					minx=i[0]
				if (i[0] > maxx):
					maxx=i[0]
			
			delta=maxx - smaxx 
			a.setLocation (a.LocX - delta, a.LocY, a.LocZ) 
			printNewCoordinates(a)
	Blender.Redraw()

#align left of objects to left of last object
def alignObjectLeft2Left():
	checkSelection(2)
	o = Blender.Object.GetSelected()[0]
	sbb=o.getBoundBox(1)
	#calc miny and max y for last object
	sminx=50000
	smaxx=-50000
	for i in sbb:
		if (i[0] < sminx):
			sminx=i[0]
		if (i[0] > smaxx):
			smaxx=i[0]
	
	# for each other objs calc its minx and maxx
	# calc delta between obj and last obj
	# set objs locx to locx - delta
	objects = Blender.Object.GetSelected()
	for a in objects:
		if ( a != o):
			bb=a.getBoundBox(1)
			minx=50000
			maxx=-50000
			for i in bb:
				if (i[0] < minx):
					minx=i[0]
				if (i[0] > maxx):
					maxx=i[0]
			
			delta=minx - sminx 
			a.setLocation (a.LocX - delta, a.LocY, a.LocZ) 
			printNewCoordinates(a)
	Blender.Redraw()

#align right of objects to left of last object
def alignObjectRight2Left():
	checkSelection(2)
	o = Blender.Object.GetSelected()[0]
	sbb=o.getBoundBox(1)
	#calc miny and max y for last object
	sminx=50000
	smaxx=-50000
	for i in sbb:
		if (i[0] < sminx):
			sminx=i[0]
		if (i[0] > smaxx):
			smaxx=i[0]
	
	# for each other objs calc its minx and maxx
	# calc delta between obj and last obj
	# set objs locx to locx - delta
	objects = Blender.Object.GetSelected()
	for a in objects:
		if ( a != o):
			bb=a.getBoundBox(1)
			minx=50000
			maxx=-50000
			for i in bb:
				if (i[0] < minx):
					minx=i[0]
				if (i[0] > maxx):
					maxx=i[0]
			
			delta=maxx - sminx 
			a.setLocation (a.LocX - delta, a.LocY, a.LocZ) 
			printNewCoordinates(a)
	Blender.Redraw()

#------------------------------------------------------------------------

#align front of objects to back of last object
def alignObjectFront2Back():
	checkSelection(2)
	o = Blender.Object.GetSelected()[0]
	sbb=o.getBoundBox(1)
	#calc minz and max z for last object
	sminz=50000
	smaxz=-50000
	for i in sbb:
		if (i[2] < sminz):
			sminz=i[2]
		if (i[2] > smaxz):
			smaxz=i[2]
	
	# for each other objs calc its minz and maxz
	# calc delta between obj and last obj
	# set objs locx to locx - delta
	objects = Blender.Object.GetSelected()
	for a in objects:
		if ( a != o):
			bb=a.getBoundBox(1)
			minz=50000
			maxz=-50000
			for i in bb:
				if (i[2] < minz):
					minz=i[2]
				if (i[2] > maxz):
					maxz=i[2]
			
			delta=maxz - sminz 
			a.setLocation (a.LocX , a.LocY, a.LocZ -delta) 
			printNewCoordinates(a)
	Blender.Redraw()

#align back of objects to back of last object
def alignObjectBack2Back():
	checkSelection(2)
	o = Blender.Object.GetSelected()[0]
	sbb=o.getBoundBox(1)
	#calc minz and max z for last object
	sminz=50000
	smaxz=-50000
	for i in sbb:
		if (i[2] < sminz):
			sminz=i[2]
		if (i[2] > smaxz):
			smaxz=i[2]
	
	# for each other objs calc its minz and maxz
	# calc delta between obj and last obj
	# set objs locx to locx - delta
	objects = Blender.Object.GetSelected()
	for a in objects:
		if ( a != o):
			bb=a.getBoundBox(1)
			minz=50000
			maxz=-50000
			for i in bb:
				if (i[2] < minz):
					minz=i[2]
				if (i[2] > maxz):
					maxz=i[2]
			
			delta=minz - sminz 
			a.setLocation (a.LocX , a.LocY, a.LocZ -delta) 
			printNewCoordinates(a)
	Blender.Redraw()

#align front of objects to front of last object
def alignObjectFront2Front():
	checkSelection(2)
	o = Blender.Object.GetSelected()[0]
	sbb=o.getBoundBox(1)
	#calc minz and max z for last object
	sminz=50000
	smaxz=-50000
	for i in sbb:
		if (i[2] < sminz):
			sminz=i[2]
		if (i[2] > smaxz):
			smaxz=i[2]
	
	# for each other objs calc its minz and maxz
	# calc delta between obj and last obj
	# set objs locx to locx - delta
	objects = Blender.Object.GetSelected()
	for a in objects:
		if ( a != o):
			bb=a.getBoundBox(1)
			minz=50000
			maxz=-50000
			for i in bb:
				if (i[2] < minz):
					minz=i[2]
				if (i[2] > maxz):
					maxz=i[2]
			
			delta=maxz - smaxz 
			a.setLocation (a.LocX , a.LocY, a.LocZ -delta) 
			printNewCoordinates(a)
	Blender.Redraw()

#align front of objects to back of last object
def alignObjectBack2Front():
	checkSelection(2)
	o = Blender.Object.GetSelected()[0]
	sbb=o.getBoundBox(1)
	#calc minz and max z for last object
	sminz=50000
	smaxz=-50000
	for i in sbb:
		if (i[2] < sminz):
			sminz=i[2]
		if (i[2] > smaxz):
			smaxz=i[2]
	
	# for each other objs calc its minz and maxz
	# calc delta between obj and last obj
	# set objs locx to locx - delta
	objects = Blender.Object.GetSelected()
	for a in objects:
		if ( a != o):
			bb=a.getBoundBox(1)
			minz=50000
			maxz=-50000
			for i in bb:
				if (i[2] < minz):
					minz=i[2]
				if (i[2] > maxz):
					maxz=i[2]
			
			delta=minz - smaxz 
			a.setLocation (a.LocX , a.LocY, a.LocZ -delta) 
			printNewCoordinates(a)
	Blender.Redraw()
#------------------------------------------------------------------------
def getnum(num):
	if ( num > 0 ):
		return 0.5
	if ( num < 0 ):
		return -0.5


def resetObjectsSize():
	checkSelection(1)
	objects = Blender.Object.GetSelected()
	print "\nReset Size X/Y/Z OF " , objects
	for a in objects:
		c = Mesh.Get(a.getData(1,0))
		print "Name=",a.getData(1,0)
		for v in c.verts:
			print "x,y,z=",v.co[0],", ",v.co[1], ", ", v.co[2]
			v.co[0] = getnum(v.co[0])
			v.co[1] = getnum(v.co[1])
			v.co[2] = getnum(v.co[2])
		a.setSize(1,1,1)
		a.setEuler(0,0,0)
		c.update()
	Blender.Redraw()


#------------------------------------------------------------------------
def Cursor2Top():
	checkSelection(1)
	objects = Blender.Object.GetSelected()[0]
	bb=objects.getBoundBox(1)
	minx=50000
	maxx=-50000
	miny=50000
	maxy=-50000
	minz=50000
	maxz=-50000
	for i in bb:
		print "x=", i[0], ", y=", i[1], ", z=", i[2]
		if (i[0] < minx):
			minx=i[0]
		if (i[1] < miny):
			miny=i[1]
		if (i[2] < minz):
			minz=i[2]
		if (i[0] > maxx):
			maxx=i[0]
		if (i[1] > maxy):
			maxy=i[1]
		if (i[2] > maxz):
			maxz=i[2]
	print 
	halfx= (maxx - minx) /2
	halfy= (maxy - miny) /2
	halfz= (maxz - minz) /2
	print "x=", minx + halfx, ", y=", miny + halfy, ", z=",maxz
 	SetCursorPos(minx + halfx, miny + halfy, maxz)
	Redraw(-1)

def Cursor2Bottom():
	checkSelection(1)
	objects = Blender.Object.GetSelected()[0]
	bb=objects.getBoundBox(1)
	minx=50000
	maxx=-50000
	miny=50000
	maxy=-50000
	minz=50000
	maxz=-50000
	for i in bb:
		print "x=", i[0], ", y=", i[1], ", z=", i[2]
		if (i[0] < minx):
			minx=i[0]
		if (i[1] < miny):
			miny=i[1]
		if (i[2] < minz):
			minz=i[2]
		if (i[0] > maxx):
			maxx=i[0]
		if (i[1] > maxy):
			maxy=i[1]
		if (i[2] > maxz):
			maxz=i[2]
	print 
	halfx= (maxx - minx) /2
	halfy= (maxy - miny) /2
	halfz= (maxz - minz) /2
	print "x=", minx + halfx, ", y=", miny + halfy, ", z=",maxz
 	SetCursorPos(minx + halfx, miny + halfy, minz)
	Redraw(-1)

def Cursor2Left():
	checkSelection(1)
	objects = Blender.Object.GetSelected()[0]
	bb=objects.getBoundBox(1)
	minx=50000
	maxx=-50000
	miny=50000
	maxy=-50000
	minz=50000
	maxz=-50000
	for i in bb:
		print "x=", i[0], ", y=", i[1], ", z=", i[2]
		if (i[0] < minx):
			minx=i[0]
		if (i[1] < miny):
			miny=i[1]
		if (i[2] < minz):
			minz=i[2]
		if (i[0] > maxx):
			maxx=i[0]
		if (i[1] > maxy):
			maxy=i[1]
		if (i[2] > maxz):
			maxz=i[2]
	print 
	halfx= (maxx - minx) /2
	halfy= (maxy - miny) /2
	halfz= (maxz - minz) /2
	print "x=", minx + halfx, ", y=", miny + halfy, ", z=",maxz
 	SetCursorPos(minx , miny + halfy, minz + halfz)
	Redraw(-1)

def Cursor2Right():
	checkSelection(1)
	objects = Blender.Object.GetSelected()[0]
	bb=objects.getBoundBox(1)
	minx=50000
	maxx=-50000
	miny=50000
	maxy=-50000
	minz=50000
	maxz=-50000
	for i in bb:
		print "x=", i[0], ", y=", i[1], ", z=", i[2]
		if (i[0] < minx):
			minx=i[0]
		if (i[1] < miny):
			miny=i[1]
		if (i[2] < minz):
			minz=i[2]
		if (i[0] > maxx):
			maxx=i[0]
		if (i[1] > maxy):
			maxy=i[1]
		if (i[2] > maxz):
			maxz=i[2]
	print 
	halfx= (maxx - minx) /2
	halfy= (maxy - miny) /2
	halfz= (maxz - minz) /2
	print "x=", minx + halfx, ", y=", miny + halfy, ", z=",maxz
 	SetCursorPos(maxx , miny + halfy, minz + halfz)
	Redraw(-1)

def Cursor2Front():
	checkSelection(1)
	objects = Blender.Object.GetSelected()[0]
	bb=objects.getBoundBox(1)
	minx=50000
	maxx=-50000
	miny=50000
	maxy=-50000
	minz=50000
	maxz=-50000
	for i in bb:
		print "x=", i[0], ", y=", i[1], ", z=", i[2]
		if (i[0] < minx):
			minx=i[0]
		if (i[1] < miny):
			miny=i[1]
		if (i[2] < minz):
			minz=i[2]
		if (i[0] > maxx):
			maxx=i[0]
		if (i[1] > maxy):
			maxy=i[1]
		if (i[2] > maxz):
			maxz=i[2]
	print 
	halfx= (maxx - minx) /2
	halfy= (maxy - miny) /2
	halfz= (maxz - minz) /2
	print "x=", minx + halfx, ", y=", miny + halfy, ", z=",maxz
 	SetCursorPos(minx + halfx, maxy , maxz + halfz)
	Redraw(-1)

def Cursor2Back():
	checkSelection(1)
	objects = Blender.Object.GetSelected()[0]
	bb=objects.getBoundBox(1)
	minx=50000
	maxx=-50000
	miny=50000
	maxy=-50000
	minz=50000
	maxz=-50000
	for i in bb:
		print "x=", i[0], ", y=", i[1], ", z=", i[2]
		if (i[0] < minx):
			minx=i[0]
		if (i[1] < miny):
			miny=i[1]
		if (i[2] < minz):
			minz=i[2]
		if (i[0] > maxx):
			maxx=i[0]
		if (i[1] > maxy):
			maxy=i[1]
		if (i[2] > maxz):
			maxz=i[2]
	print 
	halfx= (maxx - minx) /2
	halfy= (maxy - miny) /2
	halfz= (maxz - minz) /2
	print "x=", minx + halfx, ", y=", miny + halfy, ", z=",maxz
 	SetCursorPos(minx + halfx, miny , maxz + halfz)
	Redraw(-1)
#------------------------------------------------------------------------

dy=0
dx=0

#------------------------------------------------------------------------
def draw():
	global dx, dy
	def decDXBy( i):
		"Decrement dx by i, return dx"
		global dx
		ldx=dx
		dx=dx-i
		return ldx

	def decDYBy( i):
		"Decrement dx by i, return dx"
		global dy
		ldy=dy
		dy=dy-i
		return ldy

	def incDXBy( i):
		"Increment dx by i, return dx"
		global dx
		ldx=dx
		dx=dx+i
		return ldx

	def incDYBy( i):
		"Increment dx by i, return dx"
		global dy
		ldy=dy
		dy=dy+i
		return ldy

	dy=500
	dx=30
	glClearColor(.7, .7, .7, 0.0)
	glClear(GL_COLOR_BUFFER_BIT)
	glRasterPos2i(dx,dy)
	dy-=20
	Text("AlignIT Script.")
	glRasterPos2i(dx,dy)
	dy-=20
	Text("To align, select 2 or more objects, click any Align button.")
	glRasterPos2i(dx,dy)
	dy-=30
	Text("Note: Alignment uses bounding boxes. Last selected object stays put.")

	glRasterPos2i(dx,dy)
	Text("Align Buttons.")
	spcx=5
	dy -=30
	dx=30
	bw=160
	Button("Y- to Y+ of last",      72,  incDXBy(bw+spcx), dy, bw, 25)
	print "dx=", dx
	Button("Y+ to Y+ of last",      73,  incDXBy(bw+spcx), dy, bw, 25)
	Button("Y- to Y- of last",      74,  incDXBy(bw+spcx), dy, bw, 25)
	Button("Y+ to Y- of last",      75,  incDXBy(bw+spcx), dy, bw, 25)

	dy -=30
	dx=30
	Button("X- to X+ of last",      76, incDXBy(bw+spcx), dy, bw, 25)
	Button("X+ to X+ of last",      77, incDXBy(bw+spcx), dy, bw, 25)
	Button("X- to X- of last",      78, incDXBy(bw+spcx), dy, bw, 25)
	Button("X+ to X- of last",      79, incDXBy(bw+spcx), dy, bw, 25)

	dy -=30
	dx=30
	Button("Z+ to Z- of last",      80,  incDXBy(bw+spcx), dy, bw, 25)
	Button("Z- to Z- of last",      81,  incDXBy(bw+spcx), dy, bw, 25)
	Button("Z+ to Z+ of last",      82,  incDXBy(bw+spcx), dy, bw, 25)
	Button("Z- to Z+ of last",      83,  incDXBy(bw+spcx), dy, bw, 25)

	dy -=50
	dx=30
	glRasterPos2i(dx,dy)
	Text("Misc Buttons.")
	dy -=30
	bw=80
	Button("Cursor to Z+",                  85,  incDXBy(bw+spcx), dy, bw, 25)
	Button("Cursor to Z-",                  86,  incDXBy(bw+spcx), dy, bw, 25)
	Button("Cursor to X-",                  87,  incDXBy(bw+spcx), dy, bw, 25)
	Button("Cursor to X+",                  88,  incDXBy(bw+spcx), dy, bw, 25)
	Button("Cursor to Y+",                  89,  incDXBy(bw+spcx), dy, bw, 25)
	Button("Cursor to Y-",                  90,  incDXBy(bw+spcx), dy, bw, 25)

	dy -=30
	dx=30
	bw=200
	Button("Reset to 1,1,1 & scale=1 & rot=0",    84, incDXBy(bw+spcx), dy, bw, 25)

	glRasterPos2i(30,130)
	Button("Exit. Alt-q",           99, 330, 100,  100, 25)

def event(evt, val):	
	if ((evt == XKEY or evt == QKEY) and not val):
		print "Aligner/Displacer Script exited by user."
		Exit()

def bevent(evt):
	if evt == 99:
		print "Aligner/Displacer Script exited by user."
		Exit()

	elif evt == 72:
		alignObjectBot2Top()
	elif evt == 73:
		alignObjectTop2Top()
	elif evt == 74:
		alignObjectBot2Bot()
	elif evt == 75:
		alignObjectTop2Bot()

	elif evt == 76:
		alignObjectLeft2Right()
	elif evt == 77:
		alignObjectRight2Right()
	elif evt == 78:
		alignObjectLeft2Left()
	elif evt == 79:
		alignObjectRight2Left()

	elif evt == 80:
		alignObjectFront2Back()
	elif evt == 81:
		alignObjectBack2Back()
	elif evt == 82:
		alignObjectFront2Front()
	elif evt == 83:
		alignObjectBack2Front()

	elif evt == 84:
		resetObjectsSize()

	elif evt == 85:
		Cursor2Top()
	elif evt == 86:
		Cursor2Bottom()
	elif evt == 87:
		Cursor2Left()
	elif evt == 88:
		Cursor2Right()
	elif evt == 89:
		Cursor2Front()
	elif evt == 90:
		Cursor2Back()

	elif evt == 97:
		return
	Blender.Redraw()

Register(draw, event, bevent)
